<?php

// class AtomDoc is the representation in entries of an Atom xml file
// as received from a webservice. This typically handles just one entry,
// not an entire feed, but there's nothing preventing an entire feed from
// being captured.

class AtomDoc {

	// Array of all the entries found in the document
	var $entries = array();
	// feed level author
	var $author;
	// feed level contributor
	var $contributor = array();
	

	// Variables used by the parsing process
	// Element nesting stack
	var $elementStack = array();
	// current open element being parsed
	var $currentElement = "";
	// current Entry class being populated
	var $entry;
	// keep track of the current contributor	
	var $currentContributor;

	// Parsing flags
	var $authorFlag = "";
	var $contentFlag = "";
	var $entryFlag = "";


	function AtomDoc($xmlstring="") {
		if ($xmlstring) { $this->parse($xmlstring); }
	}
	

	function startNewContributor() {
		if ($this->currentContributor) {
			array_push($this->contributor, $this->currentContributor);
		}
		$this->currentContributor = new AtomAuthor();
	}
	
	function startNewAuthor() {
		$this->author = new AtomAuthor();
	}

	function closeEntry() {
		if ($this->currentContributor) {
			array_push($this->contributor, $this->currentContributor);
			$this->currentContributor="";
		}
	}


	function parse($xmlstring="") {
		$this->currentNode=0;
		$this->parentNode=0;
		
		if ($xmlstring) {
			// Set up a SAX parser to parse our xmlstring
			$parser = xml_parser_create();
			xml_set_object($parser, $this);
			xml_parser_set_option($parser, XML_OPTION_CASE_FOLDING, false);
			xml_set_element_handler($parser, "startElement", "endElement");
			xml_set_character_data_handler($parser, "characterData");

			// Start the parsing process
			if (!xml_parse($parser, $xmlstring)) {
				printf("XML error: %s at line %d while parsing entity %s\n",
					xml_error_string(xml_get_error_code($parser)),
					xml_get_current_line_number($parser), $openEntityNames);
					xml_parser_free($parser);
				return false;
			}
			xml_parser_free($parser);
			
		}
		return true;
	}
	
	// SAX Start Element event handler
	function startElement($parser, $name, $attrs) {
		// New node encountered.
		// * Push current element onto stack
		// * update current element with the new element name
		array_push ($this->elementStack, $this->currentElement);
		$this->currentElement = $name;

		if ($this->currentElement == "entry") {
			#print "EntryCond:[" . $this->authorFlag . "][" . $this->contentFlag . "][" . $this->entryFlag . "]\n";
			$this->entry = new AtomEntry();
			$this->entryFlag = "entry";

			// Prefill the author and contributor with feed level elements
			if ($this->author) {
				$this->entry->author = $this->author;
			}
			if ($this->contributor) {
				$this->entry->contributor = $this->contributor;
			}
		} elseif($this->currentElement == "content") {
			$this->entry->startNewContent();
			if ($attrs["type"]) {
				$this->entry->currentContent->type = $attrs["type"];
			}
			if ($attrs["mode"]) {
				$this->entry->currentContent->mode = $attrs["mode"];
			}
			$this->contentFlag = "content";
		} elseif ($this->currentElement == "author") {
			if ($this->entryFlag) {
				$this->entry->startNewAuthor();
				$this->authorFlag = "author";
			} else {
				$this->startNewAuthor();
				$this->authorFlag="author";
			}
		} elseif ($this->currentElement == "contributor") {
			if ($this->entryFlag) {
				$this->entry->startNewContributor();
				$this->authorFlag = "contributor";
			} else {
				$this->startNewContributor();
				$this->authorFlag = "contributor";
			}
		}
		
	}
	
	// SAX End Element event handler
	function endElement($parser, $name) {
		// End of current Node
		// * Tidy up any element work
		#print ">>Closing [" . $name . "] current [" . $this->currentElement . "]\n";
		
		if ($this->currentElement == "entry") {
			#print "\tEntry.close\n";
			$this->entry->closeEntry();
			array_push($this->entries, $this->entry);
			$this->entryFlag = "";
		} elseif ($this->currentElement == "author") {
			#print "\tAuthor.close\n";
			$this->authorFlag = "";
		} elseif ($this->currentElement == "contributor") {
			#print "\tContributor.close\n";
			$this->authorFlag = "";
		} elseif ($this->currentElement == "content") {
			#print "\tContent.close\n";
			$this->contentFlag = "";
		}
		
		// * Update current element from the elementStack
		$this->currentElement = array_pop($this->elementStack);
	}
	
	// SAX character data event handler
	function characterData($parser, $data) {
		// Check if its just whitespace
		if (!preg_match("/^\s+$/", $data)) {
			$data = trim($data);

			// The Entry Elements
			if ($this->currentElement == "title") {
				$this->entry->title .= $data;
			}
			if ($this->currentElement == "link") {
				$this->entry->link .= $data;
			}
			if ($this->currentElement == "modified") {
				$this->entry->modified .= $data;
			}
			if ($this->currentElement == "id") {
				$this->entry->id .= $data;
			}
			if ($this->currentElement == "issued") {
				$this->entry->issued .= $data;
			}
			if ($this->currentElement == "created") {
				$this->entry->created .= $data;
			}
			if ($this->currentElement == "summary") {
				$this->entry->summary .= $data;
			}
			
			// Author Elements
			if ($this->authorFlag == "author") {
				if ($this->entryFlag == "entry") {
					if ($this->currentElement == "name") {
						$this->entry->author->name .= $data;
					}
					if ($this->currentElement == "url") {
						$this->entry->author->url .= $data;
					}
					if ($this->currentElement == "email") {
						$this->entry->author->email .= $data;
					}
				} else {
					if ($this->currentElement == "name") {
						$this->author->name .= $data;
					}
					if ($this->currentElement == "url") {
						$this->author->url .= $data;
					}
					if ($this->currentElement == "email") {
						$this->author->email .= $data;
					}
				}
			}

			// Contributor Elements
			if ($this->authorFlag == "contributor") {
				if ($this->entryFlag == "entry") {
					if ($this->currentElement == "name") {
						$this->entry->currentContributor->name .= $data;
					}
					if ($this->currentElement == "url") {
						$this->entry->currentContributor->url .= $data;
					}
					if ($this->currentElement == "email") {
						$this->entry->currentContributor->email .= $data;
					}
				} else {
					if ($this->currentElement == "name") {
						$this->currentContributor->name .= $data;
					}
					if ($this->currentElement == "url") {
						$this->currentContributor->url .= $data;
					}
					if ($this->currentElement == "email") {
						$this->currentContributor->email .= $data;
					}				
				}
			}
			
			// Content
			if ($this->contentFlag == "content") {
				$this->entry->currentContent->text .= $data . "\n";
			}

		}
	}
	
	
}

// class AtomEntry holds the details of an entry

class AtomEntry {
	// Defined as per Mark Nottingham's specification at
	// http://www.mnot.net/drafts/draft-nottingham-atom-format-00.html
	
	// title - required 1
	var $title = "";

	// permalink - required 1
	var $link = "";

	// Modified date - optional 1  - W3C Date Time String UTC
	var $modified = "";

	// Author object - required 1
	var $author = "";
	
	// 1 or more contributor objects (identical to Author class)
	var $contributor = array();
	
	// Permanently global unique identifier - 1 required
	var $id = "";
	
	// Issued Date - 1 required
	var $issued = "";
	
	// Created Date - optional 1
	var $created = "";
	
	// Summary - optional 1
	var $summary = "";
	
	// content - 1 or more content objects
	var $content = array();


	// currentContent pointer
	var $currentContent;
	// current Contributor pointer
	var $currentContributor;
	
	
	// Database related fields
	var $shortName = "";
	var $entryid;
	
	
	function startNewContent() {
		if ($this->currentContent) {
			array_push($this->content, $this->currentContent);
		}
		$this->currentContent = new AtomContent();
	}

	function startNewContributor() {
		if ($this->currentContributor) {
			array_push($this->contributor, $this->currentContributor);
		}
		$this->currentContributor = new AtomAuthor();
	}
	
	function startNewAuthor() {
		$this->author = new AtomAuthor();
	}
	
	function closeEntry() {
		global $req, $urlPrefix;
		
		if ($this->currentContent) {
			array_push($this->content, $this->currentContent);
			$this->currentContent="";
		}
		if ($this->currentContributor) {
			array_push($this->contributor, $this->currentContributor);
			$this->currentContributor="";
		}
		if (!$this->shortName) {
			$this->createShortName();
		}
	}
	
	function createShortName() {
		#print "[" . $this->title . "]\n";
		$short = ucwords(strtolower($this->title));
		#print "[" . $short . "]\n";
		$this->shortName = preg_replace("/\W+/","", $short);
		#print "[" . $this->shortName . "]\n";
	}
	
	function newBlogEntryAsSql($authorid=0) {
		global $req, $urlPrefix;
		
		$this->link = $urlPrefix . "/" . $req->blogName . "/" . $this->shortName;
		if (!$this->id) {
			$this->id = $urlPrefix . "/" . $req->blogName . "/" . $this->shortName;
		}

		$sql = "INSERT INTO atomentry VALUES (null, ";

		$sql .= "'" . addslashes($this->title) . "', ";
		$sql .= "'" . addslashes($this->shortName) . "', ";
		$sql .= "'" . addslashes($this->link) . "', ";
		if ($this->modified) {
			$sql .= dateW3DateToSql($this->modified) . ", ";
		} else {
			$sql .= "null, ";
		}
		if ($authorid) {
			$sql .= $authorid . ", ";
		} else {
			$sql .= "null, ";
		}
		$sql .= "'" . addslashes($this->id) . "', ";
		if ($this->issued) {
			$sql .= dateW3DateToSql($this->issued) . ", ";
		} else {
			$sql .= "null, ";		
		}
		if ($this->created) {
			$sql .= dateW3DateToSql($this->created) . ", ";
		} else {
			$sql .= "null, ";		
		}
		$sql .= "'" . addslashes($this->summary) . "', ";
		
		$sql .= "'" . addslashes($req->blogName) . "'";

		$sql .= ")";
		return $sql;
	}
	
	function newBlogContentAsSql($entryid=0) {
		$sql = "";
		foreach ($this->content as $cont) {
			$sql .= "INSERT INTO atomcontent VALUES (null, ";
			$sql .= "'" . addslashes($cont->type) . "', ";
			#$sql .= "'', ";
			$sql .= "'" . addslashes($cont->mode) . "', ";
			#$sql .= "'', ";
			$sql .= "'" . addslashes($cont->text) . "', ";
			#$sql .= "'', ";
			if ($entryid) {
				$sql .= $entryid;
			} else {
				$sql .= "null";
			}
			$sql .= ");";
		}
		#print "SQL: " . $sql . "\n\n";
		return $sql;
	}
	
	function newBlogAuthorAsSql() {
		$sql  = "INSERT INTO atomauthor VALUES (null, ";
		$sql .=  "'" . addslashes($this->author->name) . "', ";
		$sql .=  "'" . addslashes($this->author->url) . "', ";
		$sql .=  "'" . addslashes($this->author->email) . "'";
		$sql .= ")";
		
		return $sql;
	}
	
	
	function updateBlogEntryAsSql($authorid=0) {
		global $req, $urlPrefix;
		
		$this->link = $urlPrefix . "/" . $req->blogName . "/" . $this->shortName;
		if (!$this->id) {
			$this->id = $urlPrefix . "/" . $req->blogName . "/" . $this->shortName;
		}

		$sql = "UPDATE atomentry SET ";

		$sql .= "title='" . addslashes($this->title) . "', ";
		#$sql .= "'" . addslashes($this->shortName) . "', ";
		#$sql .= "link='" . addslashes($this->link) . "', ";
		if ($authorid) {
			$sql .= "authorid=" . $authorid . ", ";
		}
		if ($this->created) {
			$sql .= "created=" . dateW3DateToSql($this->created) . ", ";
		} else {
			$sql .= "created=created, ";		
		}
		$sql .= "summary='" . addslashes($this->summary) . "'";
		
		return $sql;
	}
	
	
	function entryAsXml() {
		$xml = "\t<entry>\n";
		$xml .= "\t\t<title>" . $this->title . "</title>\n";
		$xml .= "\t\t<link>" . $this->link . "</link>\n";
		$xml .= "\t\t<id>" . $this->id . "</id>\n\n";

		if ($this->author) {
			$xml .= $this->author->authorAsXml();
			$xml .= "\n";
		}

		
		if ($this->contributor) {
			foreach($this->contributor as $contrib) {
				$xml .= $contrib->contributorAsXml();
			}
			$xml .= "\n";
		}

		if ($this->created) {
			$xml .= "\t\t<created>" . $this->created . "</created>\n";
		}
		$xml .= "\t\t<issued>" . $this->issued . "</issued>\n";
		if ($this->modified) {
			$xml .= "\t\t<modified>" . $this->modified . "</modified>\n";
		}
		$xml .= "\n";
		
		if ($this->summary) {
			$xml .= "<summary>" . $this->summary . "</summary>\n\n";
		}

		foreach ($this->content as $cont) {
			$xml .= $cont->contentAsXml();
		}

		$xml .= "\t</entry>\n";
		return $xml;
	}
	
}

class AtomContent {
	// required media-type attribute of the content
	var $type;
	
	// optional mode attribute
	var $mode;
	
	// content itself
	var $text = "";
	
	function contentAsXml() {
		$xml = "\t\t<content type=\"" . $this->type . "\"";
		if ($this->mode) {
			$xml .= " mode=\"" . $this->mode . "\"";
		}
		$xml .= ">\n";
		
		$xml .= "\t\t<![CDATA[\n";
		$xml .= $this->text;
		$xml .= "\t\t]]>\n";
		
		$xml .= "\t\t</content>\n\n";
		return $xml;
	}
	
}

class AtomAuthor {
	// Name - 1 required
	var $name = "";
	
	// author url - 1 optional
	var $url = "";
	
	// author email address - 1 optional
	var $email = "";
	
	function dataAsXml() {
		$xml = "\t\t\t<name>" . $this->name . "</name>\n";
		if ($this->url) {
			$xml .= "\t\t\t<url>" . $this->url . "</url>\n";
		}
		if ($this->email) {
			$xml .= "\t\t\t<email>" . $this->email . "</email>\n";
		}
		return $xml;
	}
	
	function authorAsXml() {
		$xml = "\t\t<author>\n";
		
		$xml .= $this->dataAsXml();
		
		$xml .= "\t\t</author>\n\n";
		return $xml;
	}

	function contributorAsXml() {
		$xml = "\t\t<contributor>\n";
		
		$xml .= $this->dataAsXml();
		
		$xml .= "\t\t</contributor>\n";
		return $xml;
	}
}


?>